@@ -21,16 +21,17 @@
 
 (* This file is for client-side comet-programming. *)
 
-open Eliom_lib
+open Js_of_ocaml
 module Ecb = Eliom_comet_base
 
-let section = Lwt_log.Section.make "eliom:comet"
+let section = Eliom_lib.Lwt_log.Section.make "eliom:comet"
+
 
 module Configuration =
 struct
   type configuration_data =
       { active_until_timeout : bool;
-        time_between_request_unfocused : (float * float) list option;
+        time_between_request_unfocused : (float * float * float) list option;
         (* (a, b) for a * t + b
            (0, 0) means always active
            None means: no request
@@ -42,7 +43,7 @@ struct
 
   let default_configuration =
     { active_until_timeout = false;
-      time_between_request_unfocused = (Some [(0.0078125, 0.)]);
+      time_between_request_unfocused = (Some [(0.5, 60., 600.)]);
       time_after_unfocus = 180.;
       time_between_request = 0.; }
 
@@ -113,7 +114,7 @@ struct
 
   let set_always_active conf v =
     set_fun conf (fun c -> { c with time_between_request_unfocused =
-        if v then Some [0., 0.] else None })
+        if v then Some [0., 0., 0.] else None })
 
   let set_timeout conf v =
     set_fun conf (fun c -> { c with time_after_unfocus = v })
@@ -131,39 +132,62 @@ struct
     let time = Sys.time () in
     let sleep_duration () = if is_idle ()
       then (match (get ()).time_between_request_unfocused, focused () with
-        | Some ((a, b)::l), Some start ->
-          let now = Js.to_float (jsnew Js.date_now ())##getTime() in
-          let t = (now -. start) *. 0.001 in (* time from idle start *)
-          let v = a *. t +. b in
-          let v = List.fold_left (fun v (a, b) -> min v (a *. t +. b)) v l in
+        | Some ((a, b, c)::l), Some start ->
+          let now = Js.to_float ((new%js Js.date_now))##getTime in
+          (* time from idle start *)
+          let t = max 0. ((now -. start) *. 0.001
+                          -. (get ()).time_after_unfocus) in
+          let v = min (a *. t +. b) c in
+          let v =
+            List.fold_left
+              (fun v (a, b, c) -> min v (min (a *. t +. b) c)) v l in
           v
         | _ -> 0. (* Configuration changed.
                      We do not sleep and we'll see later. (?) *))
       else (get ()).time_between_request
     in
     let rec aux t =
-      lwt () = Lwt.pick [Lwt_js.sleep t;
+      let%lwt () = Lwt.pick [Js_of_ocaml_lwt.Lwt_js.sleep t;
                          !update_configuration_waiter;
                          (active_waiter ())] in
       let remaining_time = sleep_duration () -. (Sys.time () -. time) in
       if remaining_time > 0.
       then aux remaining_time
-      else Lwt.return ()
+      else Lwt.return_unit
     in
     let sleep_duration = sleep_duration () in
     if sleep_duration <= 0.
-    then Lwt.return ()
+    then Lwt.return_unit
     else aux sleep_duration
 
 end
 
 
 exception Restart
-exception Process_closed
 exception Channel_closed
 exception Channel_full
 exception Comet_error of string
 
+let handle_exn, set_handle_exn_function =
+  let closed = ref false in
+  let r = ref (fun ?exn () ->
+    let s = "Unknown exception during comet. \
+             Customize this with Eliom_comet.set_handle_exn_function. "
+    in
+    match exn with
+    | Some exn -> Eliom_lib.Lwt_log.raise_error ~section ~exn s
+    | None -> Eliom_lib.Lwt_log.debug ~section s)
+  in
+  ((fun ?exn () ->
+     if not !closed
+     then begin
+       closed := true;
+       !r ?exn ()
+     end
+     else Lwt.return_unit),
+   (fun f -> r := f))
+
+
 type chan_id = string
 
 type stateless_message = ( chan_id * (string * int) Ecb.channel_data ) list
@@ -185,8 +209,6 @@ module Service_handler : sig
   (** Returns the messages received in the last request. If the
       channel is stateless, it also returns the message number in the [int option] *)
 
-  val set_activity : 'a t -> [ `Active | `Inactive | `Idle ] -> unit
-
   val activate : 'a t -> unit
 
   val is_active : 'a t -> [ `Active | `Inactive | `Idle ]
@@ -197,7 +219,6 @@ module Service_handler : sig
 
   val add_channel_stateless : stateless t -> chan_id -> Ecb.stateless_kind -> unit
 
-  val stop_waiting : stateful t -> chan_id -> unit
   val close : 'a t -> chan_id -> unit
 
 end =
@@ -215,9 +236,9 @@ struct
         mutable active_waiter : unit Lwt.t;
         (** [active_waiter] terminates when the page get focused *)
         mutable active_wakener : unit Lwt.u;
-        mutable restart_waiter : unit list Lwt.t;
-        mutable restart_wakener : unit list Lwt.u;
-        mutable active_channels : String.Set.t;
+        mutable restart_waiter : Ecb.answer Lwt.t;
+        mutable restart_wakener : Ecb.answer Lwt.u;
+        mutable active_channels : Eliom_lib.String.Set.t;
       }
 
   type stateful_state = int ref (* id of the next request *)
@@ -227,7 +248,7 @@ struct
     position : Ecb.position;
   }
 
-  type stateless_state = (stateless_state_ String.Table.t) ref (* index for each channel of the last message *)
+  type stateless_state = (stateless_state_ Eliom_lib.String.Table.t) ref (* index for each channel of the last message *)
 
   type channel_state =
     | Stateful_state of stateful_state
@@ -252,20 +273,21 @@ struct
         hd_activity : activity;
       }
 
-  let add_focus_listener f : unit =
-    let listener = Dom_html.handler (fun _ ->
-      f (); Js.bool false) in
-    (Js.Unsafe.coerce (Dom_html.window))##addEventListener(Js.string "focus",listener,Js.bool false)
+  let add_listener target event f =
+    let listener = Dom_html.handler (fun _ -> f (); Js._true) in
+    ignore @@ Dom_html.(addEventListener target event listener Js._false)
 
-  let add_blur_listener f : unit =
-    let listener = Dom_html.handler (fun _ ->
-      f (); Js.bool false) in
-    (Js.Unsafe.coerce (Dom_html.window))##addEventListener(Js.string "blur",listener,Js.bool false)
+  let add_focus_listener f = Dom_html.(add_listener window Event.focus f)
+
+  let add_blur_listener f = Dom_html.(add_listener window Event.blur f)
+
+  let add_visibility_change_listener f =
+    Dom_html.(add_listener document (Event.make "visibilitychange") f)
 
   let set_activity hd v =
-    if String.Set.is_empty hd.hd_activity.active_channels
+    if Eliom_lib.String.Set.is_empty hd.hd_activity.active_channels
     then hd.hd_activity.active <- `Inactive
-    else
+    else if v <> hd.hd_activity.active then
       match v with
         | `Inactive -> hd.hd_activity.active <- `Inactive
         | _ ->
@@ -278,23 +300,78 @@ struct
 
   let is_active hd = hd.hd_activity.active
 
+  let document_hidden () =
+    Js.Optdef.case (Js.Unsafe.global##.document##.hidden)
+      (fun () -> false) Js.to_bool
+
+  let has_focus () =
+    not (Js.Optdef.test (Js.Unsafe.global##.document##.hasFocus))
+      ||
+    Js.to_bool (Js.Unsafe.global##.document##hasFocus)
+
   (** register callbacks to 'blur' and 'focus' events of the root
       window. That way, we can tell when the client is active or not and do
       calls to the server only if it is active *)
   let handle_focus handler =
-    let focus_callback () =
-      handler.hd_activity.focused <- None;
-      set_activity handler `Active
+    let resume_activity () =
+      if handler.hd_activity.focused <> None then begin
+        handler.hd_activity.focused <- None;
+        set_activity handler `Active
+      end
     in
-    let blur_callback () =
-      handler.hd_activity.focused <-
-        Some (Js.to_float (jsnew Js.date_now ())##getTime())
+    let suspend_activity () =
+      if handler.hd_activity.focused = None then
+        handler.hd_activity.focused <-
+          Some (new%js Js.date_now)##getTime
+    in
+    let focus_callback () =
+      if not (document_hidden ()) then resume_activity ()
+    in
+    let blur_callback = suspend_activity in
+    let visibility_change_callback () =
+      if document_hidden () then
+        suspend_activity ()
+      else
+        if has_focus () then resume_activity ()
     in
     add_focus_listener focus_callback;
-    add_blur_listener blur_callback
+    add_blur_listener blur_callback;
+    add_visibility_change_listener visibility_change_callback
+
+  let expected_activity hd =
+    match hd.hd_activity.focused with
+    | None ->
+      `Active
+    | Some t ->
+      let tbru =
+        (Configuration.get ()).Configuration.time_between_request_unfocused
+      in
+      if tbru = Some [0., 0., 0.] (* Always active *) then
+        `Active
+      else
+        let now = (new%js Js.date_now)##getTime in
+        if now -. t <
+           (Configuration.get ()).Configuration.time_after_unfocus *. 1000.
+        then
+          `Active
+        else if tbru = None (* Always inactive when idle *) then
+          `Inactive
+        else
+          `Idle
 
   let activate hd =
-    set_activity hd `Active
+    if hd.hd_activity.active = `Inactive then begin
+      (* Set initial focus status *)
+      if
+        hd.hd_activity.focused = None &&
+        (document_hidden () || not (has_focus ()))
+      then
+        hd.hd_activity.focused <-
+          Some ((new%js Js.date_now)##getTime
+                -. (Configuration.get ()).Configuration.time_after_unfocus
+                   *. 1000.);
+      set_activity hd (expected_activity hd)
+    end
 
   let restart hd =
     let act = hd.hd_activity in
@@ -305,22 +382,31 @@ struct
     Lwt.wakeup_exn wakener Restart;
     activate hd
 
-  let max_retries = 5
+  let max_retries = 10
+  let initial_delay = 0.5 (* seconds *)
+  let jitter = 0.25
+  let multiplier = sqrt 2.
+  let max_delay = 4.  (* seconds *)
+  let delay i =
+    (min max_delay (initial_delay *. (multiplier ** float i)))
+     *. (1. +. jitter *. (Random.float 2. -. 1.))
 
   let call_service_after_load_end service p1 p2 =
-    lwt () = Eliom_client.wait_load_end () in
+    let%lwt () = Eliom_client.wait_load_end () in
     Eliom_client.call_service service p1 p2
 
   let make_request hd =
     match hd.hd_state with
-      | Stateful_state count -> (Ecb.Stateful (Ecb.Request_data !count))
-      | Stateless_state map ->
-        let l = String.Table.fold (fun channel { position } l -> (channel,position)::l) !map [] in
-        Ecb.Stateless (Array.of_list l)
+    | Stateful_state count -> (Ecb.Stateful (Ecb.Request_data !count))
+    | Stateless_state map ->
+      let l = Eliom_lib.String.Table.fold
+          (fun channel { position } l -> (channel,position)::l) !map []
+      in
+      Ecb.Stateless (Array.of_list l)
 
   let stop_waiting hd chan_id =
-    hd.hd_activity.active_channels <- String.Set.remove chan_id hd.hd_activity.active_channels;
-    if String.Set.is_empty hd.hd_activity.active_channels
+    hd.hd_activity.active_channels <- Eliom_lib.String.Set.remove chan_id hd.hd_activity.active_channels;
+    if Eliom_lib.String.Set.is_empty hd.hd_activity.active_channels
     then set_activity hd `Inactive
 
   let update_stateful_state hd message =
@@ -330,7 +416,7 @@ struct
         List.iter (function
           | ( chan_id, Ecb.Data _ ) -> ()
           | ( chan_id, Ecb.Closed ) ->
-            Lwt_log.ign_warning ~section
+            Eliom_lib.Lwt_log.ign_warning ~section
               "update_stateful_state: received Closed: should not happen, this is an eliom bug, please report it"
           | ( chan_id, Ecb.Full ) ->
             stop_waiting hd chan_id) message
@@ -350,6 +436,11 @@ struct
       | Ecb.After i -> i
       | Ecb.Last _ -> 0
 
+  let close_all_channels hd =
+    let s = hd.hd_activity.active_channels in
+    Eliom_lib.String.Set.iter (fun chan_id -> stop_waiting hd chan_id) s;
+    Eliom_lib.String.Set.fold (fun chan_id l -> (chan_id, None, Ecb.Closed) :: l) s []
+
   let update_stateless_state hd (message:stateless_message) =
     match hd.hd_state with
       | Stateless_state r ->
@@ -358,9 +449,9 @@ struct
             (fun table -> function
             | ( chan_id, Ecb.Data (_,index)) ->
               (try
-                 let state = String.Table.find chan_id table in
+                 let state = Eliom_lib.String.Table.find chan_id table in
                  if position_value state.position < index + 1
-                 then String.Table.add chan_id
+                 then Eliom_lib.String.Table.add chan_id
                    { state with position = (set_position state.position (index + 1)) }
                    table
                  else table
@@ -369,22 +460,26 @@ struct
             | ( chan_id, Ecb.Closed )
             | ( chan_id, Ecb.Full ) ->
               stop_waiting hd chan_id;
-              String.Table.remove chan_id table)
+              Eliom_lib.String.Table.remove chan_id table)
             !r message
         in
         r := table
       | Stateful_state _ ->
         raise (Comet_error ("update_stateless_state on stateful one"))
 
-  let call_service hd =
-    lwt () = Configuration.sleep_before_next_request
-      (fun () -> hd.hd_activity.focused)
-      (fun () -> hd.hd_activity.active = `Idle)
-      (fun () -> hd.hd_activity.active_waiter)
+  let call_service
+      ({ hd_activity; hd_service = Ecb.Comet_service srv } as hd) =
+    let%lwt () =
+      Configuration.sleep_before_next_request
+        (fun () -> hd_activity.focused)
+        (fun () -> hd_activity.active = `Idle)
+        (fun () -> hd_activity.active_waiter)
     in
     let request = make_request hd in
-    lwt s = call_service_after_load_end hd.hd_service () request in
-    Lwt.return (Ecb.Json_answer.from_string s)
+    let%lwt s =
+      call_service_after_load_end
+        srv () (hd_activity.active = `Idle, request) in
+    Lwt.return (Deriving_Json.from_string Ecb.answer_json s)
 
   let drop_message_index =
     let aux = function
@@ -403,74 +498,71 @@ struct
     List.map aux
 
   let update_activity ?(timeout=false) hd =
-    match hd.hd_activity.focused with
-      | None -> ()
-      | Some t ->
-        let tbru =
-          (Configuration.get ()).Configuration.time_between_request_unfocused
-        in
-        if not (tbru = Some [0., 0.]) (* if not always active *)
-        then begin
-          let now = Js.to_float (jsnew Js.date_now ())##getTime() in
-          if now -. t >
-            (Configuration.get ()).Configuration.time_after_unfocus *. 1000.
-          then
-            if timeout
-              || not (Configuration.get ()).Configuration.active_until_timeout
-            then set_activity hd (if tbru = None then `Inactive else `Idle)
-        end
+    if
+      hd.hd_activity.active <> `Inactive
+      &&
+      (timeout ||
+       not (Configuration.get ()).Configuration.active_until_timeout)
+    then
+      set_activity hd (expected_activity hd)
 
   let wait_data hd : (string * int option * string Ecb.channel_data) list Lwt.t =
     let rec aux retries =
       if hd.hd_activity.active = `Inactive
       then
-        lwt () = hd.hd_activity.active_waiter in
+        let%lwt () = hd.hd_activity.active_waiter in
         aux 0
       else
-	begin
-	  try_lwt
-	    lwt s = Lwt.pick [call_service hd;
-			      hd.hd_activity.restart_waiter
-			      >>= (fun _ -> assert false)] in
-	    match s with
-	      | Ecb.Timeout ->
-		update_activity ~timeout:true hd;
-		aux 0
-	      | Ecb.Process_closed -> Lwt.fail Process_closed
-	      | Ecb.Comet_error e -> Lwt.fail (Comet_error e)
-	      | Ecb.Stateless_messages l ->
+        Lwt.try_bind
+          (fun () ->
+             Lwt.pick [call_service hd;
+                       hd.hd_activity.restart_waiter])
+          (fun s ->
+            match s with
+              | Ecb.Timeout ->
+                update_activity ~timeout:true hd;
+                aux 0
+              | Ecb.State_closed ->
+                Lwt.return (close_all_channels hd)
+              | Ecb.Comet_error e -> Lwt.fail (Comet_error e)
+              | Ecb.Stateless_messages l ->
                 let l = Array.to_list l in
-		update_stateless_state hd l;
-		Lwt.return (drop_message_index l)
-	      | Ecb.Stateful_messages l ->
+                update_stateless_state hd l;
+                Lwt.return (drop_message_index l)
+              | Ecb.Stateful_messages l ->
                 let l = Array.to_list l in
-		update_stateful_state hd l;
-		Lwt.return (add_no_index l)
-          with
-            | Eliom_request.Failed_request 0 ->
+                update_stateful_state hd l;
+                Lwt.return (add_no_index l))
+          (fun e ->
+            match e with
+            | Eliom_request.Failed_request (0 | 502 | 504) ->
               if retries > max_retries
               then
-                (Lwt_log.ign_notice ~section "connection failure";
+                (Eliom_lib.Lwt_log.ign_notice ~section "connection failure";
                  set_activity hd `Inactive;
                  aux 0)
               else
-                (Lwt_js.sleep 0.05 >>= (fun () -> aux (retries + 1)))
-            | Restart -> Lwt_log.ign_info ~section "restart";
+                (let%lwt () = Js_of_ocaml_lwt.Lwt_js.sleep (delay retries) in
+                 aux (retries + 1))
+            | Restart -> Eliom_lib.Lwt_log.ign_info ~section "restart";
               aux 0
-            | exn -> Lwt_log.raise_error ~section ~exn "Error"
-        end
+            | exn ->
+              Eliom_lib.Lwt_log.ign_notice ~exn ~section "connection failure";
+              let%lwt () = handle_exn ~exn () in
+              Lwt.fail exn)
     in
     update_activity hd;
     aux 0
 
-  let call_commands hd command =
+  let call_commands {hd_service = Ecb.Comet_service srv} command =
     ignore
-      (try_lwt
-          call_service_after_load_end hd.hd_service ()
-            (Ecb.Stateful (Ecb.Commands command))
+      (try%lwt
+         call_service_after_load_end srv ()
+           (false, Ecb.Stateful (Ecb.Commands command))
        with
-         | exn -> Lwt_log.ign_notice_f ~section ~exn "request failed";
-           Lwt.return "")
+       | exn ->
+         Eliom_lib.Lwt_log.ign_notice_f ~section ~exn "request failed";
+         Lwt.return "")
 
   let close hd chan_id =
     match hd.hd_state with
@@ -479,17 +571,17 @@ struct
         call_commands hd [|Ecb.Close chan_id|]
       | Stateless_state map ->
         try
-          let state = String.Table.find chan_id !map in
+          let state = Eliom_lib.String.Table.find chan_id !map in
           if state.count = 1
-          then map := String.Table.remove chan_id !map
-          else map := String.Table.add chan_id
+          then map := Eliom_lib.String.Table.remove chan_id !map
+          else map := Eliom_lib.String.Table.add chan_id
             { state with count = state.count - 1 } !map
         with
           | Not_found ->
-            Lwt_log.ign_info_f ~section "trying to close a non existent channel: %s" chan_id
+            Eliom_lib.Lwt_log.ign_info_f ~section "trying to close a non existent channel: %s" chan_id
 
   let add_channel_stateful hd chan_id =
-    hd.hd_activity.active_channels <- String.Set.add chan_id hd.hd_activity.active_channels;
+    hd.hd_activity.active_channels <- Eliom_lib.String.Set.add chan_id hd.hd_activity.active_channels;
     call_commands hd [|Eliom_comet_base.Register chan_id|]
 
   let min_pos = function
@@ -498,7 +590,7 @@ struct
     | Ecb.Last i, Ecb.Last j -> Ecb.Last (max i j)
     | p, Ecb.Last _ -> p
     | Ecb.Last _, p -> p
-    | _ -> Lwt_log.raise_error ~section "not corresponding position"
+    | _ -> Eliom_lib.Lwt_log.raise_error ~section "not corresponding position"
 
   let add_channel_stateless hd chan_id kind =
     let pos =
@@ -507,14 +599,14 @@ struct
         | Ecb.After_kind i -> Ecb.After i
         | Ecb.Last_kind i -> Ecb.Last i
     in
-    hd.hd_activity.active_channels <- String.Set.add chan_id hd.hd_activity.active_channels;
+    hd.hd_activity.active_channels <- Eliom_lib.String.Set.add chan_id hd.hd_activity.active_channels;
     match hd.hd_state with
       | Stateful_state _ -> assert false
       | Stateless_state map ->
         begin
           let state =
             try
-              let old_state = String.Table.find chan_id !map in
+              let old_state = Eliom_lib.String.Table.find chan_id !map in
               let pos = min_pos (old_state.position,pos) in
               { count = old_state.count + 1;
                 position = pos }
@@ -523,15 +615,10 @@ struct
                 { count = 1;
                   position = pos }
           in
-          map := String.Table.add chan_id state !map;
+          map := Eliom_lib.String.Table.add chan_id state !map;
         end;
         restart hd
 
-  let stop_waiting hd chan_id =
-    hd.hd_activity.active_channels <- String.Set.remove chan_id hd.hd_activity.active_channels;
-    if String.Set.is_empty hd.hd_activity.active_channels
-    then set_activity hd `Inactive
-
   let init_activity () =
     let active_waiter,active_wakener = Lwt.wait () in
     let restart_waiter, restart_wakener = Lwt.wait () in
@@ -540,12 +627,12 @@ struct
       focused = None;
       active_waiter; active_wakener;
       restart_waiter; restart_wakener;
-      active_channels = String.Set.empty;
+      active_channels = Eliom_lib.String.Set.empty;
     }
 
   let make hd_service hd_kind =
     let hd_state = match hd_kind with
-        | Stateless -> Stateless_state (ref String.Table.empty)
+        | Stateless -> Stateless_state (ref Eliom_lib.String.Table.empty)
         | Stateful -> Stateful_state (ref 0)
     in
     let hd = {
@@ -559,6 +646,7 @@ struct
 
 end
 
+
 type 'a handler =
     { hd_service_handler : 'a Service_handler.t;
       hd_stream : ( string * int option * string Ecb.channel_data ) Lwt_stream.t; }
@@ -566,7 +654,9 @@ type 'a handler =
 let handler_stream hd =
   Lwt_stream.map_list (fun x -> x)
     (Lwt_stream.from (fun () ->
-      lwt s = Service_handler.wait_data hd in Lwt.return (Some s)))
+       Lwt.try_bind (fun () -> Service_handler.wait_data hd)
+         (fun s -> Lwt.return_some s)
+         (fun _ -> Lwt.return_none)))
 
 let stateful_handler_table : (Ecb.comet_service, Service_handler.stateful handler) Hashtbl.t
     = Hashtbl.create 1
@@ -594,7 +684,8 @@ let get_stateless_hd (service:Ecb.comet_service) : Service_handler.stateless han
     | Not_found -> init service Service_handler.stateless stateless_handler_table
 
 let activate () =
-  let f _ { hd_service_handler } = Service_handler.set_activity hd_service_handler `Active in
+  let f _ { hd_service_handler } =
+    Service_handler.activate hd_service_handler in
   Hashtbl.iter f stateless_handler_table;
   Hashtbl.iter f stateful_handler_table
 
@@ -603,10 +694,6 @@ let restart () =
   Hashtbl.iter f stateless_handler_table;
   Hashtbl.iter f stateful_handler_table
 
-let close_stateful chan_service chan_id =
-  let { hd_service_handler } = get_stateful_hd chan_service in
-  Service_handler.close hd_service_handler (Ecb.string_of_chan_id chan_id)
-
 let close = function
   | Ecb.Stateful_channel (chan_service,chan_id) ->
     let { hd_service_handler } = get_stateful_hd chan_service in
@@ -615,7 +702,7 @@ let close = function
     let { hd_service_handler } = get_stateless_hd chan_service in
     Service_handler.close hd_service_handler (Ecb.string_of_chan_id chan_id)
 
-let unmarshal s : 'a = Eliom_unwrap.unwrap (Url.decode s) 0
+let unmarshal s : 'a = Eliom_unwrap.unwrap (Eliom_lib.Url.decode s) 0
 
 type position_relation =
   | Equal   (* stateless after channels *)
@@ -636,7 +723,7 @@ let check_and_update_position position msg_pos data =
     | No_position, None, _ -> true
     | No_position, Some _, _
     | Position _, None, Ecb.Data _ ->
-      Lwt_log.raise_error ~section "check_position: channel kind and message do not match"
+      Eliom_lib.Lwt_log.raise_error ~section "check_position: channel kind and message do not match"
     | Position _, None, (Ecb.Full | Ecb.Closed ) -> true
     | Position (relation,r), Some j, _ ->
       match !r with
@@ -664,8 +751,8 @@ let register' hd position (chan_service:Ecb.comet_service) (chan_id:'a Ecb.chan_
           | Ecb.Closed ->
             Lwt.fail Channel_closed
           | Ecb.Data x ->
-            Lwt.return (Some (unmarshal x:'a)))
-      | _ -> Lwt.return None)
+            Lwt.return_some (unmarshal x:'a))
+      | _ -> Lwt.return_none)
     (Lwt_stream.clone hd.hd_stream)
   in
   let protect_and_close t =
