@@ -28,60 +28,63 @@ type position =
   | Newest of int
   | After of int
   | Last of int option (* None means this is a 'newest channel' *)
-deriving (Json)
+[@@deriving json]
 
 type comet_stateless_request = (string*position) array
-deriving (Json)
+[@@deriving json]
 
 type command =
   | Register of string
   | Close of string
-deriving (Json)
+[@@deriving json]
 
 type comet_stateful_request =
   | Request_data of int
   | Commands of command array
-deriving (Json)
+[@@deriving json]
 
 type comet_request =
   | Stateless of comet_stateless_request
   | Stateful of comet_stateful_request
-deriving (Json)
+[@@deriving json]
 
 let comet_request_param =
-  Eliom_parameter.ocaml "comet_request" Json.t<comet_request>
+  Eliom_parameter.ocaml "comet_request" [%json: comet_request]
 
 type 'a channel_data =
   | Data of 'a
   | Full
   | Closed
-deriving (Json)
+[@@deriving json]
 
 type answer =
   | Stateless_messages of (string * (string * int) channel_data) array
   | Stateful_messages of (string * string channel_data) array
   | Timeout
-  | Process_closed
+  | State_closed
   | Comet_error of string
-deriving (Json)
+[@@deriving json]
 
 type comet_service =
-    (unit, comet_request,Eliom_service.service_method,Eliom_service.attached,
-     Eliom_service.service_kind,
-     [ `WithoutSuffix ], unit,
-     [ `One of comet_request Eliom_parameter.ocaml ] Eliom_parameter.param_name,
-     Eliom_service.registrable,
-     Eliom_registration.http_service )
-      Eliom_service.service
+    Comet_service :
+      (unit, bool * comet_request, Eliom_service.post, Eliom_service.att,
+       _, _, _, [ `WithoutSuffix ], unit,
+       [ `One of bool ] Eliom_parameter.param_name *
+       [ `One of comet_request Eliom_parameter.ocaml ]
+         Eliom_parameter.param_name,
+       Eliom_service.non_ocaml) Eliom_service.t ->
+    comet_service
 
 type internal_comet_service =
-    (unit, comet_request,Eliom_service.service_method,Eliom_service.attached,
-     Eliom_service.internal_service_kind,
-     [ `WithoutSuffix ], unit,
-     [ `One of comet_request Eliom_parameter.ocaml ] Eliom_parameter.param_name,
-     [ `Registrable ],
-     Eliom_registration.http_service )
-      Eliom_service.service
+    Internal_comet_service :
+      (unit, bool * comet_request, Eliom_service.post, Eliom_service.att,
+       _, Eliom_service.non_ext, Eliom_service.reg,
+       [ `WithoutSuffix ], unit,
+       [ `One of bool ] Eliom_parameter.param_name *
+       [ `One of comet_request Eliom_parameter.ocaml ]
+         Eliom_parameter.param_name,
+       Eliom_service.non_ocaml) Eliom_service.t ->
+    internal_comet_service
 
 type stateless_kind =
   | After_kind of int
@@ -92,15 +95,15 @@ type 'a wrapped_channel =
   | Stateful_channel of (comet_service * 'a chan_id)
   | Stateless_channel of (comet_service * 'a chan_id * stateless_kind)
 
-
 type 'a bus_send_service =
+  Bus_send_service :
     (unit,
-     'a list, [`Post], Eliom_service.non_attached_kind, [`NonattachedCoservice],
+     'a list, Eliom_service.post, Eliom_service.non_att,
+     Eliom_service.co, Eliom_service.non_ext, Eliom_service.reg,
      [ `WithoutSuffix ],
      unit,
      [ `One of 'a list Eliom_parameter.ocaml ] Eliom_parameter.param_name,
-     [ `Registrable ],
-     Eliom_registration.http_service
-    ) Eliom_service.service
+     Eliom_service.non_ocaml) Eliom_service.t ->
+  'a bus_send_service
 
 type ('a, 'b) wrapped_bus = 'b wrapped_channel * 'a bus_send_service
