@@ -12,10 +12,14 @@ let usage () =
   Printf.eprintf "  -eliom-inc <dir>\tAdd <dir> to the list of eliom include directories (prepend eliom build directories)\n";
   Printf.eprintf "  -package <name>\tRefer to package when compiling\n";
   Printf.eprintf "  -no-autoload\t\tDo not load commonly used syntax extensions (deriving, lwt, js_of_ocaml, tyxml)\n";
-  Printf.eprintf "  -type-conv\t\tUse type_conv syntax extensions instead of deriving one\n";
-  Printf.eprintf "\t\t\tIt has no effect if used in conjunction with -no-autoload\n";
 
-  Printf.eprintf "  -ppopt <p>\t\tAppend option <opt> to preprocessor invocation\n";
+  Printf.eprintf "  -ppopt <opt>\t\tAppend option <opt> to preprocessor invocation\n";
+  Printf.eprintf "  -ppx";
+  Printf.eprintf "\t\t\tUse the PPX Eliom syntax extension (default: Camlp4)\n";
+  Printf.eprintf "\t\t\tThe above description only applies to the first\n";
+  Printf.eprintf "\t\t\tappearance of -ppx in the list of flags. Subsequent\n";
+  Printf.eprintf "\t\t\tappearances require an argument and specify a\n";
+  Printf.eprintf "\t\t\tPPX preprocessor to use (see STANDARD OPTIONS).\n";
   Printf.eprintf "  -predicates <p>\tAdd predicate <p> when resolving package properties\n";
   Printf.eprintf "  -verbose\t\tPrint calls to external commands\n";
   create_filter !compiler ["-help"] (help_filter 2 "STANDARD OPTIONS:");
@@ -86,40 +90,59 @@ let eliom_synonyms = [ "-ml-synonym"; ".eliom"; "-mli-synonym"; ".eliomi" ]
 
 let compile_intf file =
   create_filter
-    !compiler ( "-pp" :: get_pp [] !ppopt :: eliom_synonyms @ !args
-        @ (map_include !eliom_inc_dirs)
-		@ ["-intf"; file] )
+    !compiler (
+    preprocess_opt ~ocaml:true !ppopt
+    @ eliom_synonyms @ !args
+    @ (map_include !eliom_inc_dirs)
+    @ ["-intf"; file] )
     (on_each_line add_build_dirs)
 
 let compile_impl file =
   create_filter
-    !compiler ( "-pp" :: get_pp [] !ppopt :: eliom_synonyms @ !args
-        @ (map_include !eliom_inc_dirs)
-		@ ["-impl"; file] )
+    !compiler (
+    preprocess_opt ~ocaml:true !ppopt
+    @ eliom_synonyms @ !args
+    @ (map_include !eliom_inc_dirs)
+    @ ["-impl"; file] )
     (on_each_line add_build_dirs)
 
 let server_pp_opt impl_intf =
-  ["-notype"] @ !ppopt @ [impl_intf_opt impl_intf]
+  let l = ["-notype"] @ !ppopt in
+  match !pp_mode with
+  | `Ppx ->
+    l
+  | _ ->
+    l @ [impl_intf_opt impl_intf]
+
 let client_pp_opt impl_intf =
-  ["-notype"] @ !ppopt @ [impl_intf_opt impl_intf]
+  let l = ["-notype"] @ !ppopt in
+  match !pp_mode with
+  | `Ppx ->
+    l
+  | _ ->
+    l @ [impl_intf_opt impl_intf]
+
 let type_pp_opt impl_intf =
-  !ppopt @ [impl_intf_opt impl_intf]
+  match !pp_mode with
+  | `Ppx ->
+    !ppopt
+  | _ ->
+    !ppopt @ [impl_intf_opt impl_intf]
 
 let compile_server_eliom ~impl_intf file =
   if !do_dump then begin
     let camlp4, ppopt =
       get_pp_dump
-        ["eliom.syntax.server"]
+        []
         ("-printer" :: "o" :: server_pp_opt impl_intf @ [file]) in
     ignore (create_process camlp4 ppopt);
     exit 0
   end;
   create_filter !compiler
-    ( "-pp" ::
-      get_pp
-        ["eliom.syntax.server"]
-        (server_pp_opt impl_intf) :: eliom_synonyms @ !args
+    ( preprocess_opt ~kind:`Server (server_pp_opt impl_intf)
+      @ eliom_synonyms @ !args
       @ (map_include !eliom_inc_dirs)
+      @ get_common_ppx ~kind:`Server ()
       @ [impl_intf_opt impl_intf; file] )
     (on_each_line add_build_dirs)
 
@@ -128,17 +151,16 @@ let compile_type_eliom ~impl_intf file =
     (* Won't run because [compile_server_eliom] is first and exits ... *)
     let camlp4, ppopt =
       get_pp_dump
-        ["eliom.syntax.type"]
+        []
         ("-printer" :: "o" :: type_pp_opt impl_intf @ [file]) in
     ignore (create_process camlp4 ppopt);
     exit 0
   end;
   create_filter !compiler
-    ( "-pp" ::
-      get_pp
-        ["eliom.syntax.type"]
-        (type_pp_opt impl_intf) :: eliom_synonyms @ !args
+    ( preprocess_opt ~kind:`Types (type_pp_opt impl_intf)
+      @ eliom_synonyms @ !args
       @ (map_include !eliom_inc_dirs)
+      @ get_common_ppx ~kind:`Server ()
       @ [impl_intf_opt impl_intf; file] )
     (on_each_line server_type_file_dependencies)
 
@@ -146,17 +168,16 @@ let compile_client_eliom ~impl_intf file =
   if !do_dump then begin
     let camlp4, ppopt =
       get_pp_dump
-        ["eliom.syntax.client"]
+        []
         ("-printer" :: "o" :: client_pp_opt impl_intf @ [file]) in
     ignore (create_process camlp4 ppopt);
     exit 0
   end;
   create_filter !compiler
-    ( "-pp" ::
-      get_pp
-        ["eliom.syntax.client"]
-        (client_pp_opt impl_intf) :: eliom_synonyms @ !args
+    ( preprocess_opt ~kind:`Client (client_pp_opt impl_intf)
+      @ eliom_synonyms @ !args
       @ (map_include !eliom_inc_dirs)
+      @ get_common_ppx ~kind:`Client ()
       @ [impl_intf_opt impl_intf; file] )
     (on_each_line add_build_dirs)
 
@@ -175,26 +196,30 @@ let compile_eliom ~impl_intf file =
       Printf.printf "%s.cmx : %s\n" (add_build_dir basename) (get_type_file file) )
 
 let sort () =
-  let pkg, ppopt =
+  let ppopt =
     match !kind with
-      | `Server | `ServerOpt ->
-          ["eliom.syntax.server"], server_pp_opt `Impl
-      | `Client -> ["eliom.syntax.client"], client_pp_opt `Impl
+      | `Server | `ServerOpt -> server_pp_opt `Impl
+      | `Client -> client_pp_opt `Impl
   in
   wait
     (create_process !compiler
-       ( "-sort" :: "-pp" :: get_pp pkg ppopt :: eliom_synonyms @
+       ( "-sort" ::
+         preprocess_opt ~kind:!kind ppopt @
+         eliom_synonyms @
+         get_common_ppx ~kind:!kind () @
+         map_include !eliom_inc_dirs @
          List.(concat (map (fun file -> ["-impl"; file]) !sort_files)) )
     ) ;
   0
 
+
 let process_option () =
-  let i = ref 2 in
+  let i = ref 2
+  and ppx = ref false in
   while !i < Array.length Sys.argv do
     match Sys.argv.(!i) with
     | "-verbose" -> verbose := true; incr i
     | "-no-autoload" -> autoload_predef := false; incr i
-    | "-type-conv" -> type_conv := true; incr i
     | "-sort" -> mode := `Sort; incr i
     | "-eliom-inc" ->
       if !i+1 >= Array.length Sys.argv then usage ();
@@ -229,6 +254,10 @@ let process_option () =
     | "-dump" ->
       do_dump := not !do_dump;
       i := !i+1
+    | "-ppx" when not !ppx ->
+      ppx := true;
+      pp_mode := `Ppx;
+      i := !i+1
     | "-intf" ->
       if !i+1 >= Array.length Sys.argv then usage ();
       let arg = Sys.argv.(!i+1) in
